<?php if (!defined('PmWiki')) exit();
/* 
   publish page collection as book script, version 2.0.18
   Copyright 2003-2007 John Rankin john.rankin@affinity.co.nz
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This file turns a trail page into a typeset book of the contents, in a
   form suitable for printing, via the (:typeset-book:) directive.
*/

function HandleBookPublish($pagename, $auth = 'read') {
  global $HandlePublishFmt,$SpaceWikiWords,$HeadingDepth,$format;
  global $PublishFormPageFmt,$HTMLHeaderFmt,$PdfFrontMatterPattern;
  global $PrintSurTitleFmt,$PrintHeaderFmt,$PrintSubTitleFmt,$PageAnchors;
  global $ExtractTrailStopsFunction,$PrintThisPageFunction,$XMLEncodingFmt;
  SDV($ExtractTrailStopsFunction, 'ExtractTrailStops');
  SDV($PrintThisPageFunction, 'PrintThisPage');
  $Tp    = 'part';  $Ta = 'appendix';
  $Tu    = $Tp.' class="unnumber"';
  $Tbook = array('chapter',
     'section','subsection','subsubsection','paragraph','subparagraph');
  $Chapter  = ($format=='pdf') ? 'CH:' : '-&gt;';
  $TbookPTV = array('dedication' => 'DED', 'graphics' => 'GRA');
  $TbookDir = array('description' => 'publisher', 'keywords' => 'keywords');
  $TitleFmt = ($SpaceWikiWords) ? '$Titlespaced' : '$Title';

  $trailname = $pagename;
  $trailpage = RetrieveAuthPage($pagename, $auth, true, READPAGE_CURRENT);
  if (!$trailpage) Abort('?cannot read $pagename');
  PCache($pagename,$trailpage);
  if ($format=='pdf') {
      Markup('tbchap','block','/^CH:(.*?)(?:\s*&mdash;\s*(.*?))?$/e',
        "'<:block><tbook:aphorism>'.PSS('$1').
        ('$2' ? '<tbook:caption>'.PSS('$2').'</tbook:caption>' : '').
        '</tbook:aphorism>'");
      Markup('tblogo','block','/^GRA:([^:]+):(.*?)$/','<:block>$2');
      Markup('tbdedn','block','/^DED:([^:]+):(.*?)$/','<:block><tbook:$1>$2</tbook:$1>');
      foreach ($TbookPTV as $k => $v)
          if (preg_match("/\\(:$k:\\s+(.*?):\\)/",$trailpage['text'],$dir))
              array_unshift($HTMLHeaderFmt, "markup:$v:$k:".$dir[1]);
      foreach ($TbookDir as $k => $v)
          if (preg_match("/\\(:$k\\s+(.*?):\\)/",$trailpage['text'],$dir))
              array_unshift($HTMLHeaderFmt,
                   "\n<$v>".htmlentities($dir[1],ENT_NOQUOTES,$XMLEncodingFmt)."</$v>");
      if (preg_match("/\\(:typeset-book\\*?\\s+(.*?):\\)/",$trailpage['text'],$pdf))
          SetPdfMetaTags($pdf[1]);
      SDVA($HTMLHeaderFmt,
          PublishFormDefaults($trailname,$PublishFormPageFmt['book']));
  }
  $FM = FrontMatter('default');
  foreach ($PdfFrontMatterPattern as $k => $v) SDV($$v, $FM[$k]);
  $trailpage['text'] = str_replace("\\\\\n",'[[&lt;&lt;]]',$trailpage['text']);
  $aphorisms = ExtractChapterAphorisms($trailname,$trailpage['text']);
  $dpl = $ExtractTrailStopsFunction($trailname,$trailpage['text']);
  foreach ($dpl as $k => $v) $$k = $v;

    $counter = array(); $predepth= 0; $appendix = false; $part = false;
    for($i=0;$i<count($pages);$i++) {
      @$counter[$depth[$i]-1]++;
      $num = $counter[0];
      for($j=$depth[$i];$j<$predepth;$j++) $counter[$j] = 0;
      $size = 0;
      for($j=1;$j<$depth[$i];$j++) { $num .= "." . $counter[$j]; $size++; }
      if ($size>3) $size=3;
      
      $anchor = "wikipublisher.$num";
      $num = "[[#$anchor]]";

      if ($format=='pdf') {
          $olink = $links[$i];
          $tbtag = '<'.$Tbook[$depth[$i]-1];
          if ($depth[$i]==1) {
            $linktxt = TextOfLink($trailname, $links[$i]);
            if (preg_match('/^[Aa]ppendix[:\s]\s*(.*)$/',$linktxt,$m)) {
              if (preg_match('/^([Uu]n)?[Pp]art:([^:]*):\s*(.*)$/',$m[1],$p)) {
                  $m[1] = $p[3];
                  $pun  = $p[1] ? $Tu : $Tp;
                  $tbtag= "<$pun>".MarkupToHTML($pagename,"H:$p[2]").$tbtag;
                  $tbtag= ($part ? "</$Tp>" : '').($appendix ? '' : "<$Ta>").$tbtag;
                  $part = true;
              } else{
                  $tbtag= ($appendix ? '' : ($part ? "</$Tp>" : '')."<$Ta>").$tbtag;
                  $part = ($appendix ? $part : false);
              }
              $olink    = str_replace($m[0],$m[1],$links[$i]);
              $appendix = true;
            } elseif (preg_match('/^((?:[Uu]n)?[Nn]umber|[Pp]reface):\s*(.*)$/',$linktxt,$m)) {
              $olink    = str_replace($m[0],$m[2],$links[$i]);
              $linktxt  = $m[1];
              if ($part) { $part = false;  $tbtag = "</$Tp>" . $tbtag; }
            } elseif (preg_match('/^([Aa]fterword)(:\s*(.*))?$/',$linktxt,$m)) {
              if ($m[2]) $olink = str_replace($m[0],$m[3],$links[$i]);
              $linktxt  = $m[1];
              if ($part) { $part = false;  $tbtag = "</$Tp>" . $tbtag; }
            } elseif (preg_match('/^([Uu]n)?[Pp]art:([^:]*):\s*(.*)$/',MarkupEscape($linktxt),$m)) {
              $m = MarkupRestore($m);
              $olink    = str_replace($m[0],$m[3],$links[$i]);
              $pun      = $m[1] ? $Tu : $Tp;
              $tbtag    = ($part ? "</$Tp>" : '')."<$pun>".MarkupToHTML($pagename,"H:$m[2]").$tbtag;
              $part     = true;
            }
            if (preg_match('/"([^"]+)"$/',$linktxt,$hdg))
                $olink = str_replace($hdg[0],'',$olink).$hdg[0];
            $tbtag .= ChapterKind(strtolower($linktxt));
          }
          $tbtag .= '>';
          for($j=$depth[$i];$j<=$predepth;$j++) 
            $tbtag = '</'.$Tbook[$j-1].'>'.$tbtag;
          $PublishList[] = $tbtag;
          $PublishList[] = "function:PrintThisMarkup H:$num$olink";
      } else {
          $s = str_repeat('-',$size);
          $h = "!%block class='pagehead$depth[$i]'%";
          if ($size==0) $PublishList[] = "markup:$h$num$links[$i]";
          else $PublishList[] = "markup:$h"."[$s$num$links[$i]$s]";
      }      
      $predepth = $depth[$i];

      if ($pages[$i]!="textonly") {
          $PageAnchors[$pages[$i]]  = $anchor;
          $HeadingDepth[$pages[$i]] = $Tbook[$depth[$i]];
          if (isset($aphorisms[$pages[$i]])) 
              $PublishList[] = "markup:$Chapter".$aphorisms[$pages[$i]];
          if (PageExists($pages[$i])) 
	       	  $PublishList[] = "function:$PrintThisPageFunction $pages[$i]";
          elseif ($depth[$i]>1) {
              $replname = TextOfLink($trailname,$links[$i]);
              $PublishList[] = "function:PrintThisText " . 
    ((MakeLink($trailname,"[[$replname]]",NULL,NULL,'$FullName')==$pages[$i]) ?
                $links[$i] : "[[".$pages[$i]."]] ($replname)");
          }
      }
    }

    if ($format=='pdf') {
        $tbtag = '';
        for($j=1;$j<=$predepth;$j++) 
            $tbtag = '</'.$Tbook[$j-1].'>'.$tbtag;
        if ($part)     $tbtag.= "</$Tp>";
        if ($appendix) $tbtag.= "</$Ta>";
        $PublishList[]  = $tbtag;
        $PublishList[0] = 
          str_replace('<chapter','<chapter id="wikipublisher.top"',$PublishList[0]);
    } else array_unshift($PublishList,'markup:[[#wikipublisher.top]]');

  $GLOBALS['PublishList'] = $PublishList;
  PrintFmt($pagename,$HandlePublishFmt);
}

function ExtractChapterAphorisms($trailname,$trailtext) {
  global $SuffixPattern; $aphorisms = array();
  foreach(explode("\n",$trailtext) as $x) {
      if (preg_match("/^:\\s*([[:upper:]\\[].*)/",$x,$match)) {
          $match[1] = ReformatTrailStop(preg_replace("/\\{(\\$.*?)\\}/e",
                "FmtPageName('$1',\$trailname)",$match[1]));
          if (preg_match("/^(\\[\\[([^|:]*?)(?:\\|([^\\[\\]]*?(?:\\[.*?\\].*?)?))?\\]\\]($SuffixPattern))\\s*:(.*)/",
          $match[1],$m)) {
              $aphorisms[MakePageName($trailname,$m[2])] = $m[5];
          }
      }
  }
  return $aphorisms;
}

function ChapterKind($kind) {
  global $ChapterKindFmt;
  $att = array('preface', 'introduction', 'acknowledgments', 'colophon',
               'prologue', 'epilogue', 'foreword', 'afterword', 'unnumber');
  for ($j=0;$j<count($att);$j++)
    if ($kind==(isset($ChapterKindFmt[$j]) ? strtolower($ChapterKindFmt[$j]) : 
               (isset($ChapterKindFmt[$att[$j]]) ? strtolower($ChapterKindFmt[$att[$j]]) : $att[$j])))
        return " kind='$att[$j]'";
  return '';
}

?>